// cert_openssl.h
//

#ifndef _CERT_OPENSSL_H_
#define _CERT_OPENSSL_H_

#include <pktlab.h>

#include <stdbool.h>

#include "util_openssl.h"

#include <openssl/ssl.h>

//
// EXPORTED TYPE DEFINITION
//

struct pktlab_certificate {
    X509 * x509;
};

//
// EXPORTED FUNCTION DECLARATIONS
//

// check fields
extern bool is_ed25519_signalg(const X509 * x509);
extern bool has_valid_signature(X509 * x509, struct pktlab_publickey * signkey);
extern bool is_ed25519_signature(const X509 * x509);
extern bool is_ver_3(const X509 * x509);
extern bool has_valid_keyusage(X509 * x509, int cert_type);
extern bool is_valid_ca_pathlen(int cert_type, bool ca, long pathlen);
extern bool are_do_not_use_fields_used(const X509 * x509);

// get field values
extern int get_serial (
    const X509 * x509,
    uint8_t ** buf, uint_fast32_t * len);
extern int get_aid(const X509 * x509, void * buf, uint_fast32_t buflen);
extern int get_sid(const X509 * x509, void * buf, uint_fast32_t buflen);
extern int get_validity (
    const X509 * x509,
    pktlab_time_t * not_valid_before,
    pktlab_time_t * not_valid_after);
extern int get_subj_pubkey(const X509 * x509, uint8_t * signkey);
extern int get_basic_constraints(X509 * x509, bool * ca, long * pathlen);
extern int get_cert_type(const X509 * x509);
extern int get_del_type(const X509 * x509, uint8_t * del_type);
extern int get_ext_ascii_str (
    const X509 * x509,
    int nid, bool expectcrit, char ** str);
extern int get_ext_digests_non_critical (
    const X509 * x509,
    int nid, uint_fast32_t digestlen,
    uint8_t ** digests, uint_fast32_t * dcnt);

#define get_filter_digests(x509,filter_digests,ftcnt) \
    get_ext_digests_non_critical( \
        (x509), _nid_filter_digests, \
        PKTLAB_FILTER_DIGEST_LEN, (filter_digests), (ftcnt))
#define get_monitor_digests(x509,monitor_digests,mtcnt) \
    get_ext_digests_non_critical( \
        (x509), _nid_monitor_digests, \
        PKTLAB_MONITOR_DIGEST_LEN, (monitor_digests), (mtcnt))
#define get_cert_desc(x509,cert_desc) get_ext_ascii_str((x509),(_nid_cert_desc),false,(cert_desc))
#define get_priority(x509,priority)   get_ext_ascii_str((x509),(_nid_priority),false,(priority))

// misc functions
extern int comp_keyid (
    const uint8_t * rawkey, uint8_t * buf);

extern int dup_cert (
    const struct pktlab_certificate * orgcert,
    struct pktlab_certificate ** copiedcert);

#endif