// ctrl_tcponly.c
//

#include <pktlab.h>
#include <pktctrl.h>

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "ctrl.h"
#include "private.h"

//
// EXPORTED FUNCTION DEFINITIONS
//

// Functions supported but different from SSL counterparts
struct pktctrl_obj * pktctrl_create_obj(void) {
	struct pktctrl_obj * obj;

	trace("%s()", __func__);

	obj = safe_malloc(sizeof(*obj));
	memset(obj, 0, sizeof(*obj));

	obj->type = NEITHER_TYPE;
	obj->state = ST_INIT;
	return obj;
}

int pktctrl_close(struct pktctrl_obj * obj) {
	trace("%s(state:%d,type:%d,sockfd:%d)",
		__func__, obj->state, obj->type, obj->sockfd);

	if (obj == NULL)
		return PKTCTRL_SUCCESS;

	if (obj->reader != NULL)
		pktlab_close_reader(obj->reader);
	if (obj->writer != NULL)
		pktlab_close_writer(obj->writer);

	// sockfd is NOT closed here
	free(obj);
	return PKTCTRL_SUCCESS;
}

// Functions not supported
struct pktctrl_keycert_store * pktctrl_create_keycert_store(void) {
	trace("%s()", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return NULL;
}

int pktctrl_set_server_keycert (
	struct pktctrl_keycert_store * store, const char * snikey,
	struct pktlab_certificate ** cert_ls, uint_fast32_t certnum,
	struct pktlab_publickey ** root_ls, uint_fast32_t rootnum)
{
	trace("%s(snikey:%s,certnum:%u,rootnum:%u)", __func__,
		(snikey == NULL) ? "(NULL)" : snikey,
		(unsigned) certnum, (unsigned) rootnum);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_get_server_keycert (
	struct pktctrl_keycert_store * store, const char * snikey,
	struct pktlab_certificate *** cert_ls, uint_fast32_t * certnum,
	struct pktlab_publickey *** root_ls, uint_fast32_t * rootnum)
{
	trace("%s(snikey:%s)", __func__, (snikey == NULL) ? "(NULL)" : snikey);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_clear_server_keycert (
	struct pktctrl_keycert_store * store, const char * snikey)
{
	trace("%s(snikey:%s)", __func__, snikey);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_set_server_keycert_default (
	struct pktctrl_keycert_store * store, const char * snikey)
{
	trace("%s(snikey:%s)", __func__, snikey);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_clear_server_keycert_default (
	struct pktctrl_keycert_store * store)
{
	trace("%s()", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_set_client_keycert (
	struct pktctrl_keycert_store * store,
	struct pktlab_certificate ** cert_ls, uint_fast32_t certnum,
	struct pktlab_publickey ** root_ls, uint_fast32_t rootnum)
{
	trace("%s(certnum:%u,rootnum:%u)", __func__,
		(unsigned) certnum, (unsigned) rootnum);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_get_client_keycert (
	struct pktctrl_keycert_store * store,
	struct pktlab_certificate *** cert_ls, uint_fast32_t * certnum,
	struct pktlab_publickey *** root_ls, uint_fast32_t * rootnum)
{
	trace("%s()", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_clear_client_keycert(struct pktctrl_keycert_store * store)
{
	trace("%s()", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_cleanup_keycert_store(struct pktctrl_keycert_store * store) {
	trace("%s()", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_accept (
	int fd, int auth_mode,
	struct pktlab_privatekey * serverkey,
	struct pktctrl_keycert_store * store, struct pktctrl_obj * ctrlobj,
	struct pktlab_certificate *** peer_cert_ls, uint_fast32_t * peer_cert_num)
{
	trace("%s(auth_mode:%d)", __func__, auth_mode);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_connect (
	int fd, int auth_mode, const char * snikey,
	struct pktlab_privatekey * clientkey,
	struct pktctrl_keycert_store * store, struct pktctrl_obj * ctrlobj,
	struct pktlab_certificate *** peer_cert_ls, uint_fast32_t * peer_cert_num)
{
	trace("%s(auth_mode:%d,snikey:%s)", __func__, auth_mode, snikey);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}

int pktctrl_gen_snikey(char ** buf) {
	trace("%s(...)", __func__);
	debug("%s not supported (SSL not enabled)", __func__);
	return PKTCTRL_UNKNOWN_FAULT;
}