// util.h
//

#ifndef _UTIL_H_
#define _UTIL_H_

#include <stddef.h>
#include <stdint.h>

#include <signal.h>
#include <pthread.h>

#include <pktlab.h>

//
// EXPORTED MACROS
//

#define MIN(a,b) (((a) < (b)) ? (a):(b))
#define MAX(a,b) (((a) > (b)) ? (a):(b))

//
// EXPORTED FUNCTION DECLARATIONS
//

extern void * pktlabme_safe_malloc_(size_t size);
extern void * pktlabme_safe_calloc_(size_t nelts, size_t eltsz);
extern void * pktlabme_safe_realloc_(void * ptr, size_t size);

extern const char * strchr_ext(const char * s, int c, size_t * l);

extern int copy_str(const char * s, size_t l, char ** copied_str);

extern int read_whole_file (
    const char * path,
    uint8_t ** content, size_t * contentlen);

extern int posix_getpass (
    const char * prompt, char ** passphrase);

extern bool is_valid_pktlab_time(const char * pt, pktlab_time_t * rst);
extern bool is_valid_ull(const char * ull, unsigned long long * rst);
extern bool is_valid_filepath(const char * path);
extern bool is_valid_iso31661_alpha3(const char * country);
extern bool is_valid_maidenhead_w_wc(const char * locator);
extern bool is_valid_hexstr(const char * hexstr);

extern int overflow_safe_add_uf32 (
    uint_fast32_t u1, uint_fast32_t u2,
    uint_fast32_t * rst);

extern int get_hex_val(char c);
extern int hexstr2bytes (
    const char * hexstr, size_t len,
    uint8_t ** bytes, size_t * blen);

extern pktlab_time_t ceil_add_time (
    pktlab_time_t i, pktlab_time_t j);

extern int set_close_on_exec(int fd);

extern void lock_until_success(pthread_mutex_t * mutex);
extern void unlock_until_success(pthread_mutex_t * mutex);

extern void set_fd_set(int * nsel, fd_set * set, int fd);

extern int set_sigmask_block_sigint(sigset_t * oset);
extern int restore_sigmask(sigset_t * oset);

//
// SHORTER NAME MACROS
//

#define safe_malloc pktlabme_safe_malloc_
#define safe_calloc pktlabme_safe_calloc_
#define safe_realloc pktlabme_safe_realloc_

#define MAX(a,b) (((a) > (b)) ? (a):(b))

#endif
