// main.c
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#include <pktlab.h>

#include "console.h"
#include "manager.h"

#define USAGE "pktendpt [-h] [-c EP_CONF_PATH] [ADDR[:PORT]]"

#define DEFAULT_MECONF_PATH "/.pktlab/endpt.conf" // prepended with HOME env var
#define FALLBACK_MECONF_PATH "endpt.conf"

#define FILEPATH_BUFSZ 4096

//
// TYPE DEFINITIONS
//

// ...

//
// RUN-TIME PARAMETERS
//
// TODO: parse more arguments: broker address, certificates file, xsub channel
// list
struct {
    const char *ctlspec;
    const char *confpath;
    const char *brokerspec;
    const char *const *certpaths;
    const char *troot;
} _args;

//
// GLOBAL VARIABLES
//

// ...

//
// INTERNAL FUNCTION DECLARATIONS
//

static void initialize(int argc, char *const argv[]);

//
// MAIN FUNCTION DEFINITION
//

int main(int argc, char *const argv[]) {
    initialize(argc, argv);

    manager_init();
    manager_start(_args.confpath, _args.ctlspec);
    manager_wait();

    exit(EXIT_SUCCESS);
    return 0;
}

//
// INTERNAL FUNCTION DEFINITIONS
//

void initialize(int argc, char *const argv[]) {
    static const char * const default_ctlspec = PKTLAB_DEFAULT_SPEC_STR;
    const char * homepath;
    size_t homepathlen;
    static char confpath_buf[FILEPATH_BUFSZ];
    struct sigaction action;
    int optch;

    // Disable SIGPIPE signal; handle broken pipe error on write.

    memset(&action, 0, sizeof(action));
    action.sa_handler = SIG_IGN;
    sigaction(SIGPIPE, &action, NULL);

    // Close stdin and stdout.

    fclose(stdin);
    fclose(stdout);

    // Parse command-line arguments.

    while ((optch = getopt(argc, argv, "c:h")) != -1) {
        switch (optch) {
        case 'c':
            _args.confpath = optarg;
            break;
        case 'h':
        default:
            goto usage_error;
        }
    }

    argc -= optind;
    argv += optind;

    // set up default option/argument values
    if (argc > 0)
        _args.ctlspec = *argv;
    else
        _args.ctlspec = default_ctlspec;

    if (_args.confpath == NULL) {
        homepath = getenv("HOME");
        if (homepath == NULL) {
            warn("Cannot get HOME environment variable value");
            warn("Looking for %s in current working directory instead",
                FALLBACK_MECONF_PATH);
            _args.confpath = FALLBACK_MECONF_PATH;
        } else {
            homepathlen = strlen(homepath);
            if (homepathlen > sizeof(confpath_buf)-sizeof(DEFAULT_MECONF_PATH)) {
                fatal("Config path buffer overflow: HOME path too long (%zu)",
                    homepathlen);
                exit(EXIT_FAILURE);
            }

            strncpy(confpath_buf, homepath, sizeof(confpath_buf));
            strncpy(
                confpath_buf+homepathlen, DEFAULT_MECONF_PATH,
                sizeof(confpath_buf)-homepathlen); // '\0' appended
            _args.confpath = confpath_buf;
        }
    }

    return;

usage_error:
    fprintf(stderr, "USAGE: " USAGE "\n");
    exit(EXIT_FAILURE);
    return;
}
