// util.h
//

#ifndef _UTIL_H_
#define _UTIL_H_

#include <stddef.h>

#include <pktlab.h>

//
// EXPORTED MACRO DEFINITIONS
//

#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#define MAX(a, b) (((a) > (b)) ? (a) : (b))

//
// EXPORTED FUNCTION DECLARATIONS
//

#define perror_exit_if(cond, error_msg)                                        \
    if (cond)                                                                  \
    perror(error_msg)

extern void *pktlabme_safe_malloc_(size_t size);
extern void *pktlabme_safe_calloc_(size_t nelts, size_t eltsz);
extern void *pktlabme_safe_realloc_(void *ptr, size_t size);

extern const char *strchr_ext(const char *s, int c, size_t *l);
extern int copy_str(const char *s, size_t l, char **copied_str);
extern bool is_valid_filepath(const char *path);
extern bool is_valid_hexstr(const char *hexstr);
extern int read_whole_file(const char *path, uint8_t **content,
                           size_t *contentlen);
extern int posix_getpass(const char *prompt, char **passphrase);
extern int overflow_safe_add_uf32(uint_fast32_t u1, uint_fast32_t u2,
                                  uint_fast32_t *rst);
extern int get_hex_val(char c);
extern int hexstr2bytes(const char *hexstr, size_t len, uint8_t **bytes,
                        size_t *blen);
//
// SHORTER NAME MACROS
//

#define safe_malloc pktlabme_safe_malloc_
#define safe_calloc pktlabme_safe_calloc_
#define safe_realloc pktlabme_safe_realloc_
#define safe_strdup pktlabme_safe_strdup_

#endif
